<?php
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Quads\Quad_BgsPlayerCard;
use FML\ManiaLink;
use FML\Script\Features\Paging;
use ManiaControl\Bills\BillData;
use ManiaControl\Bills\BillManager;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Commands\CommandListener;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;

/**
 * Created by PhpStorm.
 * User: XeRoX
 * Date: 03.06.2015
 * Time: 19:51
 */
class Bookkeeping implements Plugin, CallbackListener, CommandListener {

	/** @var  ManiaControl $maniaControl */
	private $maniaControl;

	const TABLE_BOOKKEEPING = 'mc_bookkeeping';

	const ID_A = -2;
	const ID_D = -3;

	const SETTING_MULTISERVER = 'Hide data of other servers';

	/**
	 * Prepare the Plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 */
	public static function prepare(ManiaControl $maniaControl) {
		// TODO: Implement prepare() method.
	}

	/**
	 * Load the plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 * @return bool
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_MULTISERVER, false);

		$this->maniaControl->getCommandManager()->registerCommandListener(array('bookkeeping', 'bk'), $this, 'command_bookkeeping', true, 'Seee past transactions of the server');
		$this->maniaControl->getCommandManager()->registerCommandListener(array('bookkeeping_players',
		                                                                        'bk_players'), $this, 'command_bookkeeping_players', true, 'Seee past transactions of the server');

		$this->maniaControl->getCallbackManager()->registerCallbackListener(BillManager::CB_BILL_PAID, $this, 'billPayed');

		$this->initTable();

	}

	public function billPayed($state, BillData $billData) {

		switch ($state) {
			case (BillManager::DONATED_TO_RECEIVER):
				$receiver = $this->maniaControl->getPlayerManager()->getPlayer($billData->receiverLogin);
				$this->insertToTable($billData->player, $receiver, $billData->amount);
				break;
			case (BillManager::DONATED_TO_SERVER):
				$this->insertToTable($billData->player, null, $billData->amount);

				break;
			case (BillManager::PAYED_FROM_SERVER):
				$player = $this->maniaControl->getPlayerManager()->getPlayer($billData->player);
				if (!$player) {
					$player = $billData->player;
				}
				$message = '';
				if (property_exists($billData, 'message')) {
					$message = $billData->message;
				}
				$this->insertToTable(null, $player, $billData->amount, $message);
				break;
		}
	}


	private function insertToTable($transmitterPlayer, $receiverPlayer, $amount, $message = '') {

		$receiverLogin = '';
		$server        = $this->maniaControl->getServer()->index;

		if (!$transmitterPlayer) {
			$transmitter = 0;
		} else {
			$transmitter = $transmitterPlayer->index;
		}


		if ($receiverPlayer instanceof Player) {
			$receiver      = $receiverPlayer->index;
			$receiverLogin = $receiverPlayer->login;
		} elseif ($receiverPlayer == self::ID_A || $receiverPlayer == self::ID_D) {
			$receiver = $receiverPlayer;
		} else if (is_string($receiverPlayer)) {
			$receiver      = -1;
			$receiverLogin = $receiverPlayer;
		} else {
			$receiver = 0;
		}


		$mysqli = $this->maniaControl->getDatabase()->getMysqli();
		$query  = "INSERT INTO `" . self::TABLE_BOOKKEEPING . "` (
				`transmitter`,
				`receiver`,
				`receiver_login`,
				`amount`,
				`message`,
				`server`
				) VALUES (
				{$transmitter},
				{$receiver},
				'{$receiverLogin}',
				{$amount},
				'{$message}',
				{$server}
				);";

		$mysqli->query($query);

		if ($mysqli->error) {
			trigger_error($mysqli->error);
			return false;
		}

	}


	private function initTable() {
		$mysqli = $this->maniaControl->getDatabase()->getMysqli();

		$query = "CREATE TABLE IF NOT EXISTS `" . self::TABLE_BOOKKEEPING . "` (
				`index` int(11) NOT NULL AUTO_INCREMENT,
				`transmitter` int(11) NOT NULL DEFAULT 0,
				`receiver` int(11) NOT NULL DEFAULT 0,
				`receiver_login` varchar(100) NOT NULL DEFAULT '',
				`amount` int(11) NOT NULL DEFAULT 0,
				`transaction_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				`message` varchar(250) DEFAULT '',
				`server` int(11) DEFAULT 0,
				PRIMARY KEY (`index`),
				KEY `time` (`transaction_time`)
				) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Save planet transactions' AUTO_INCREMENT=1;";

		$mysqli->query($query);
		if ($mysqli->error) {
			trigger_error($mysqli->error, E_USER_ERROR);
		}

		$query  = "SHOW COLUMNS FROM " . self::TABLE_BOOKKEEPING;
		$res    = $mysqli->query($query);
		$exists = false;
		while ($row = $res->fetch_object()) {
			if ($row->Field == 'server') {
				$exists = true;
			}
		}
		$res->free();

		if (!$exists) {
			$query = "ALTER TABLE " . self::TABLE_BOOKKEEPING . " ADD server INT(11) NOT NULL AFTER message";

			$mysqli->query($query);
			if ($mysqli->error) {
				trigger_error($mysqli->error, E_USER_ERROR);
			}
		}

		$this->insertToTable(null, self::ID_A, $this->maniaControl->getClient()->getServerPlanets());

	}

	public function command_bookkeeping(array $chatCallback, Player $player) {

		$width  = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsWidth();
		$height = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsHeight();

		$mysqli = $this->maniaControl->getDatabase()->getMysqli();

		$server    = $this->maniaControl->getServer()->index;
		$serversql = "'" . $server . "'";

		if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MULTISERVER)) {
			$query = 'SELECT * FROM ' . self::TABLE_BOOKKEEPING . ' WHERE server =' . $serversql . '
             ORDER BY transaction_time DESC LIMIT 500';
		} else {
			$query = 'SELECT * FROM ' . self::TABLE_BOOKKEEPING . '
             ORDER BY transaction_time DESC LIMIT 500';
		}
		if ($mysqli->error) {
			trigger_error($mysqli->error, E_USER_ERROR);
		}
		$result = $mysqli->query($query);

		// create manialink
		$maniaLink = new ManiaLink(ManialinkManager::MAIN_MLID);
		$script    = $maniaLink->getScript();
		$paging    = new Paging();
		$script->addFeature($paging);

		// Main frame
		$frame = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultListFrame($script, $paging);
		$maniaLink->add($frame);

		// Start offsets
		$posX = -$width / 2;
		$posY = $height / 2;

		//Predefine description Label
		$descriptionLabel = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultDescriptionLabel();
		$frame->add($descriptionLabel);

		// Headline
		$headFrame = new Frame();
		$frame->add($headFrame);
		$headFrame->setY($posY - 5);
		if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MULTISERVER)) {
			$array = array('$oId' => $posX + 5, '$oTransmitter' => $posX + 10, '$oReceiver' => $posX + 50, '$oAmount' => $posX + 90, '$oTransaction time' => $posX + 110);
		} else {
			$array = array('$oId' => $posX + 5, '$oTransmitter' => $posX + 10, '$oReceiver' => $posX + 35, '$oAmount' => $posX + 60, '$oTransaction time' => $posX + 90, '$oServer' => $posX + 130);
		}
		$this->maniaControl->getManialinkManager()->labelLine($headFrame, $array);

		$index     = 1;
		$posY      = $posY - 10;
		$pageFrame = null;

		while ($row = $result->fetch_object()) {

			if ($index % 16 === 1) {
				$pageFrame = new Frame();
				$frame->add($pageFrame);
				$posY = $height / 2 - 10;
				$paging->addPage($pageFrame);
			}

			$playerFrame = new Frame();
			$pageFrame->add($playerFrame);
			$playerFrame->setY($posY);

			if ($index % 2 !== 0) {
				$lineQuad = new Quad_BgsPlayerCard();
				$playerFrame->add($lineQuad);
				$lineQuad->setSize($width, 4);
				$lineQuad->setSubStyle($lineQuad::SUBSTYLE_BgPlayerCardBig);
				$lineQuad->setZ(0.001);
			}

			if ($row->transmitter == 0) {
				$transmitter = 'Server';
			} elseif ($row->transmitter == -1) {
				$transmitter = 'Unknown Player';
			} elseif ($row->transmitter == 0) {
				$transmitter = 0;
			} else {
				$transmitter = $this->maniaControl->getPlayerManager()->getPlayerByIndex($row->transmitter)->nickname;
			}
			if ($row->receiver == 0) {
				$receiver = 'Server';
			} elseif ($row->receiver == self::ID_A || $row->receiver == self::ID_D) {
				$receiver = 0;
			} elseif ($row->receiver == -1) {
				$receiver = $row->receiver_login;
			} else {
				$receiver = $this->maniaControl->getPlayerManager()->getPlayerByIndex($row->receiver)->nickname;
			}

			$servers     = $this->maniaControl->getServer()->getAllServers();
			$serverlogin = "";
			$serverID    = $row->server;

			foreach ($servers as $server) {
				if ($server->index == $serverID) {
					$serverlogin = $server->login;
					break;
				}
			}

			if ($row->receiver == self::ID_A) {
				if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MULTISERVER)) {
					$positions = array($posX + 5, $posX + 10, $posX + 70, $posX + 110);
					$texts     = array($index, '$5b5Plugin activated', 'Balance: ' . $row->amount, $row->transaction_time);
				} else {
					$positions = array($posX + 5, $posX + 10, $posX + 60, $posX + 90, $posX + 130);
					$texts     = array($index, '$5b5Plugin activated', 'Balance: ' . $row->amount, $row->transaction_time, $serverlogin);
				}
			} elseif ($row->receiver == self::ID_D) {
				if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MULTISERVER)) {
					$positions = array($posX + 5, $posX + 10, $posX + 70, $posX + 110);
					$texts     = array($index, '$f43Plugin deactivated', 'Balance: ' . $row->amount, $row->transaction_time);
				} else {
					$positions = array($posX + 5, $posX + 10, $posX + 60, $posX + 90, $posX + 130);
					$texts     = array($index, '$f43Plugin deactivated', 'Balance: ' . $row->amount, $row->transaction_time, $serverlogin);
				}
			} else {
				if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MULTISERVER)) {
					$texts     = array($index, $transmitter, $receiver, $row->amount, $row->transaction_time);
					$positions = array($posX + 5, $posX + 10, $posX + 50, $posX + 90, $posX + 110);
				} else {
					$texts     = array($index, $transmitter, $receiver, $row->amount, $row->transaction_time, $serverlogin);
					$positions = array($posX + 5, $posX + 10, $posX + 35, $posX + 60, $posX + 90, $posX + 130);
				}
			}

			$this->maniaControl->getManialinkManager()->labelLine($playerFrame, array($positions, $texts));

			$posY -= 4;
			$index++;
		}
		$this->maniaControl->getManialinkManager()->sendManialink($maniaLink, $player);
	}

	public function command_bookkeeping_players(array $chatCallback, Player $player) {

		$width  = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsWidth();
		$height = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsHeight();

		$players = $this->maniaControl->getPlayerManager()->getPlayers();

		$mysqli = $this->maniaControl->getDatabase()->getMysqli();

		$server    = $this->maniaControl->getServer()->index;
		$serversql = "'" . $server . "'";

		$query = 'SELECT * FROM ' . self::TABLE_BOOKKEEPING . ' WHERE (server =' . $serversql . ' OR server = 0) AND receiver != ' . self::ID_A .' AND receiver != ' . self::ID_D . '
             ORDER BY transaction_time DESC';

		$result = $mysqli->query($query);

		foreach ($players as $player) {
			$player->paid            = 0;
			$player->received        = 0;
			$player->lastTransaction = 0;
		}

		while ($row = $result->fetch_object()) {

			if ($row->receiver == $player->index) {
				$player->received += $row->amount;
				if ($player->lastTransaction < $row->transaction_time) {
					$player->lastTransaction = $row->transaction_time;
				}
			} elseif ($row->transmitter == $player->index) {
				$player->paid += $row->amount;
				if ($player->lastTransaction < $row->transaction_time) {
					$player->lastTransaction = $row->transaction_time;
				}
			}
		}


		// create manialink
		$maniaLink = new ManiaLink(ManialinkManager::MAIN_MLID);
		$script    = $maniaLink->getScript();
		$paging    = new Paging();
		$script->addFeature($paging);

		// Main frame
		$frame = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultListFrame($script, $paging);
		$maniaLink->add($frame);

		// Start offsets
		$posX = -$width / 2;
		$posY = $height / 2;

		//Predefine description Label
		$descriptionLabel = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultDescriptionLabel();
		$frame->add($descriptionLabel);

		// Headline
		$headFrame = new Frame();
		$frame->add($headFrame);
		$headFrame->setY($posY - 5);

		$array = array('$oId' => $posX + 5, '$oPlayer' => $posX + 10, '$oPlanets received' => $posX + 60, '$oPlanets paid' => $posX + 90, '$oLast Transaction' => $posX + 110);

		$this->maniaControl->getManialinkManager()->labelLine($headFrame, $array);

		$index     = 1;
		$posY      = $posY - 10;
		$pageFrame = null;

		foreach ($players as $player) {

			if ($index % 15 === 1) {
				$pageFrame = new Frame();
				$frame->add($pageFrame);
				$posY = $height / 2 - 10;
				$paging->addPage($pageFrame);
			}

			$playerFrame = new Frame();
			$pageFrame->add($playerFrame);
			$playerFrame->setY($posY);

			if ($index % 2 !== 0) {
				$lineQuad = new Quad_BgsPlayerCard();
				$playerFrame->add($lineQuad);
				$lineQuad->setSize($width, 4);
				$lineQuad->setSubStyle($lineQuad::SUBSTYLE_BgPlayerCardBig);
				$lineQuad->setZ(0.001);
			}

			$texts     = array($index, $player->nickname, $player->received, $player->paid, $player->lastTransaction);
			$positions = array($posX + 5, $posX + 10, $posX + 60, $posX + 90, $posX + 110);

			$this->maniaControl->getManialinkManager()->labelLine($playerFrame, array($positions, $texts));

			$posY -= 4;
			$index++;

		}
		$this->maniaControl->getManialinkManager()->sendManialink($maniaLink, $player);
	}

	/**
	 * Unload the plugin and its Resources
	 */
	public function unload() {
		$this->insertToTable(null, self::ID_D, $this->maniaControl->getClient()->getServerPlanets());
	}

	/**
	 * Get plugin id
	 *
	 * @return int
	 */
	public static function getId() {
		return 77;
	}

	/**
	 * Get Plugin Name
	 *
	 * @return string
	 */
	public static function getName() {
		return 'Bookkeeping';
	}

	/**
	 * Get Plugin Version
	 *
	 * @return string
	 */
	public static function getVersion() {
		return '0.33';
	}

	/**
	 * Get Plugin Author
	 *
	 * @return string
	 */
	public static function getAuthor() {
		return 'XeRoX';
	}

	/**
	 * Get Plugin Description
	 *
	 * @return string
	 */
	public static function getDescription() {
		return 'Logs every Planet transaction on the server';
	}
}